/*------------------------------------------------
--                 LAZZE-GETOBZ                 --
--------------------------------------------------
-- Filename      : game.c
-- Description   : Misc game stuff
--                 
--
-- Author        : Lars Englund
--                 d99las@dtek.chalmers.se
--
--                 Tobias Gedell
--                 d99getob@dtek.chalmers.se
--
-- Created       : 020831
-- Last update   : 020902
------------------------------------------------*/


#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "robot.h"
#include "error.h"
#include "game.h"
#include "gdebug.h"
#include "map.h"


int robotNumPackets;
packetStruct **robotPackets;

int numGamePackets;
packetStruct *pfirstGamePacket=NULL;
packetStruct *plastGamePacket=NULL;

int numPlayers;
playerStruct *pfirstPlayer=NULL;
playerStruct *plastPlayer=NULL;
playerStruct *probot=NULL;

int moveCount=0;


void addPlayer(int x, int y, int id)
{
    //- Don't add our robot if probot!=NULL
    if(id==robotId&&probot!=NULL) return;

    //- First player?
    if( pfirstPlayer==NULL )
    {
        if((pfirstPlayer = (playerStruct*)malloc( sizeof(playerStruct) ))==NULL)
            ERROR("Cannot allocate player");

        plastPlayer = pfirstPlayer;
        plastPlayer->prev = NULL;
    }
    else
    {
        if((plastPlayer->next = (playerStruct*)malloc( sizeof(playerStruct) ))==NULL)
            ERROR("Cannot allocate player");
        plastPlayer->next->prev=plastPlayer;
        plastPlayer = plastPlayer->next;
    }
    plastPlayer->id = id;
    plastPlayer->x = x;
    plastPlayer->y = y;
    plastPlayer->numPackets = 0;
    plastPlayer->timeStamp = moveCount;
    plastPlayer->next = NULL;
    if( id == robotId )
    {
        probot = plastPlayer;
        robotNumPackets = plastPlayer->numPackets;
        robotPackets = &plastPlayer->packets[0];
        
        robotX = x;
        robotY = y;
    }

    numPlayers++;
}


void removePlayer( int id )
{
    playerStruct *tmp = pfirstPlayer;
    while( tmp )
    {
        if( tmp->id == id )
        {
            if(tmp==pfirstPlayer) pfirstPlayer=tmp->next;
            if(tmp==plastPlayer) plastPlayer=tmp->prev;

            if(tmp->prev) tmp->prev->next=tmp->next;
            if(tmp->next) tmp->next->prev=tmp->prev;

            free(tmp);

            numPlayers--;

            #ifdef DEBUG
            printf( "Player %d removed..\n", id );
            #endif

            return;
        }
        tmp = tmp->next;
    }
    ERROR("Invalid player ID in removePlayer");
}


void removeDeadPlayers()
{
    playerStruct *tmp = pfirstPlayer;
    while( tmp )
    {
        if( tmp->timeStamp != moveCount )
        {
            #ifdef DEBUG
            printf( "Player %d has died!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! Removing..\n", tmp->id );
            #endif
            removePlayer( tmp->id );
        }
        tmp = tmp->next;
    }
}


void timeStampPlayer( int playId )
{
    playerStruct *tmp = pfirstPlayer;
    while( tmp )
    {
        if( tmp->id == playId )
        {
            tmp->timeStamp = moveCount;
            return;
        }
        tmp = tmp->next;
    }
}


void movePlayer( int playId, int dir )
{
    playerStruct *tmp = pfirstPlayer;
    while( tmp )
    {
        if( tmp->id == playId )
        {
            switch( dir )
            {
                //- NOTE: We should be able to remove the wall test
                case LEFT:  if( WALL == getGamePoint( --(tmp->x), tmp->y ) ) tmp->x++;
                            break;
                case UP:    if( WALL == getGamePoint( tmp->x, ++(tmp->y) ) ) tmp->y--;
                            break;
                case RIGHT: if( WALL == getGamePoint( ++(tmp->x), tmp->y ) ) tmp->x--;
                            break;
                case DOWN:  if( WALL == getGamePoint( tmp->x, --(tmp->y) ) ) tmp->y++;
                            break;
                default:
                    ERROR("Invalid direction in movePlayer");
            }
            #ifdef DEBUG
//            printf("Player %d pos after move: %d, %d\n", playId, tmp->x, tmp->y);
            #endif

            if( playId == robotId )
            {
                robotX = tmp->x;
                robotY = tmp->y;
            }
                
            return;
        }
        tmp = tmp->next;
    }
    ERROR("Invalid player ID in movePlayer");
}


packetStruct *getPacketPointer( int packId )
{
    packetStruct *tmp = pfirstGamePacket;
    while( tmp )
    {
        if( tmp->id == packId ) return tmp;
        tmp = tmp->next;
    }
    return NULL;
}


void removePacket( int id )
{
    packetStruct *tmp = pfirstGamePacket;
    while( tmp )
    {
        if( tmp->id == id )
        {
            if(tmp==pfirstGamePacket) pfirstGamePacket=tmp->next;
            if(tmp==plastGamePacket) plastGamePacket=tmp->prev;
            
            if(tmp->prev) tmp->prev->next=tmp->next;
            if(tmp->next) tmp->next->prev=tmp->prev;

            free(tmp);

            numGamePackets--;
            
            #ifdef DEBUG
            printf("Packet %d removed and deallocated\n", id);
            #endif
            
            return;
        }
        tmp = tmp->next;
    }
    ERROR("Invalid packet ID in removePacket");
}


packetStruct *addPacket( int id, int destX, int destY, int weight )
{
    // Check if packet is already known
    packetStruct *tmp = pfirstGamePacket;
    while( tmp )
    {
        if( tmp->id == id )
        {
            tmp->destX = destX;
            tmp->destY = destY;
            tmp->weight = weight;
            tmp->timeStamp=moveCount;
            return tmp;
        }
        tmp = tmp->next;
    }
    
    // Create new packet
    if( pfirstGamePacket==NULL )
    {
        if((pfirstGamePacket = (packetStruct*)malloc( sizeof(packetStruct) ))==NULL)
            ERROR("Cannot allocate packet");
        plastGamePacket = pfirstGamePacket;
        plastGamePacket->prev = NULL;
    }
    else
    {
        if((plastGamePacket->next = (packetStruct*)malloc( sizeof(packetStruct) ))==NULL)
            ERROR("Cannot allocate packet");
        plastGamePacket->next->prev=plastGamePacket;
        plastGamePacket = plastGamePacket->next;
    }
    plastGamePacket->id = id;
    plastGamePacket->destX = destX;
    plastGamePacket->destY = destY;
    plastGamePacket->x = robotX;
    plastGamePacket->y = robotY;
    plastGamePacket->weight = weight;
    //- SCORING
    plastGamePacket->score=-1;
    plastGamePacket->timeStamp=moveCount;
    plastGamePacket->next = NULL;

    numGamePackets++;
    
    return plastGamePacket;
}


void playerPick( int playId, int packId )
{
    packetStruct *pack;
    playerStruct *tmp = pfirstPlayer;
    
    while( tmp )
    {
        if( tmp->id == playId )
        {
            // Check if the packet ID is known
            pack = getPacketPointer( packId );
            if( pack )
            {
                //- Add packet to player
                tmp->packets[tmp->numPackets++] = pack;
                
                //- Check if robot is us
                if(tmp->id==robotId)
                {
                    robotNumPackets=tmp->numPackets;
                    robotCapacity-=pack->weight;

                    //- Add dest to info map
                    putDestPoint(pack->destX, pack->destY, 1);

                    //- Since we have a packet we do not want more
                    robotLookingForPackets=0;
                }
                
                //- Dec packet count
                decPacketPoint(tmp->x, tmp->y);

                //- Set packet pos to (-1, -1)
                pack->x=-1;
                pack->y=-1;
                
                #ifdef DEBUG
                printf("Player %d got packet %d\n", playId, packId);
                #endif
            }

            return;
        }
        tmp = tmp->next;
    }
    ERROR("Invalid player ID in playerPick");
}


void playerDrop( int playId, int packId )
{
    int n;
    playerStruct *tmp = pfirstPlayer;
    packetStruct *pack;
    
    while(tmp)
    {
        if(tmp->id == playId)
        {
            //- Player found, check if he has the packet
            for(n=0; n<tmp->numPackets; n++)
            {
                pack=tmp->packets[n];
                
                if(pack->id == packId)
                {
                    //- Packet found, set position to robot position
                    pack->x=tmp->x;
                    pack->y=tmp->y;
                    //- SCORING
                    pack->score=-1;
                    
                    //- Check if player is us
                    if(tmp->id==robotId)
                    {
                        robotCapacity+=tmp->packets[n]->weight;
                    }

                    // Remove packet if dropped on destination 
                    if(tmp->x==pack->destX && tmp->y==pack->destY) removePacket(packId);
                    else
                    {
                        //- Inc packet count
                        incPacketPoint(tmp->x, tmp->y);
                    }
                                    
                    //- Remove packet from player
                    memmove(&(tmp->packets[n]), &(tmp->packets[n+1]), sizeof(playerStruct*) * (tmp->numPackets-n-1));
                        
                    tmp->numPackets--;
                    
                    //- Check if player is us
                    if(tmp->id==robotId)
                    {
                        robotNumPackets=tmp->numPackets;

                        putDestPoint(tmp->x, tmp->y, 0);

                        //- If we don't have any packets left
                        //- we want to look for packets
                        if(robotNumPackets<1) robotLookingForPackets=1;
                    }
                    
                    
                    return;
                }
            }
            //- Packet not found, insert packet with unknown dest and weight
            pack = addPacket( packId, -1, -1, -1 );
            pack->x = tmp->x;
            pack->y = tmp->y;
            //- SCORING
            pack->score=-1;
            
            //- Inc packet count
            incPacketPoint(tmp->x, tmp->y);

            return;
        }
        tmp = tmp->next;
    }
    ERROR("Invalid player ID in playerDrop");
}
