/*------------------------------------------------
--                 LAZZE-GETOBZ                 --
--------------------------------------------------
-- Filename      : gdebuc.h
-- Description   : Graphical debug functions
--                 
--
-- Author        : Lars Englund
--                 d99las@dtek.chalmers.se
--
--                 Tobias Gedell
--                 d99getob@dtek.chalmers.se
--
-- Created       : 020901
-- Last update   : 020902
------------------------------------------------*/


#ifdef GDEBUG

#include <SDL/SDL.h>

#include <stdio.h>

#include "gdebug.h"
#include "error.h"
#include "map.h"
#include "game.h"
#include "robot.h"

#define SCREENS         6

#define WINDOWWIDTH    600
#define WINDOWHEIGHT   600

#define SCREENWIDTH    300   
#define SCREENHEIGHT   300


SDL_Surface *window;
SDL_Surface *screens[SCREENS];

void DrawBox(int screen, int x, int y, Uint8 r, Uint8 g, Uint8 b, int width, int height);
void ShowScreens();


void Slock(SDL_Surface *screen);
void Sulock(SDL_Surface *screen);


void initGDebug()
{
    Uint32 rmask, gmask, bmask, amask;
    
    int ggr;
    int x,y;

    #ifdef DEBUG
    printf("initGDebug()\n");
    #endif

#if SDL_BYTEORDER == SDL_BIG_ENDIAN
    rmask = 0xff000000;
    gmask = 0x00ff0000;
    bmask = 0x0000ff00;
    amask = 0x000000ff;
#else
    rmask = 0x000000ff;
    gmask = 0x0000ff00;
    bmask = 0x00ff0000;
    amask = 0xff000000;
#endif

    atexit(SDL_Quit);

    if(SDL_Init(SDL_INIT_VIDEO)<0)
        ERROR("Cannot init SDL");

    if((window=SDL_SetVideoMode(WINDOWWIDTH, WINDOWHEIGHT, 32, SDL_HWSURFACE|SDL_DOUBLEBUF))==NULL)
        ERROR("Cannot create window");
    
    SDL_WM_SetCaption("LAZZE-GETOBZ", NULL);
    

    for(ggr=0;ggr<SCREENS;ggr++)
    {
        if((screens[ggr]=SDL_CreateRGBSurface(SDL_HWSURFACE, SCREENWIDTH, 
                                        SCREENHEIGHT, 32, rmask, gmask, bmask, amask))==NULL)
            ERROR("Cannot create screen");
    }
}


void DrawScreens()
{
    int ggr;
    int ggr2;
    playerStruct *player;
    packetStruct *packet;
    
    //- Lock screens
    for(ggr=0;ggr<SCREENS;ggr++)
        Slock(screens[ggr]);

    //- Draw game board
    for(ggr=0;ggr<mapWidth;ggr++)
        for(ggr2=0;ggr2<mapHeight;ggr2++)
        {
            int point;
            int r, g, b;
            
            point=getGamePoint(ggr, ggr2);
            switch(point)
            {
                case WATER:
                    r=0;
                    g=0;
                    b=200;
                    break;
                    
                case BASE:
                    r=0;
                    g=100;
                    b=0;
                    break;
                    
                case WALL:
                    r=120;
                    g=80;
                    b=0;
                    break;
                    
                case PLAIN:
                    r=170;
                    g=170;
                    b=170;
                    break;
                    
                default:
                    printf("game map: ('%c', %d)\n\n", point, point);
                    ERROR("Invalid point in DrawScreens");
            }
            
            DrawBox(0, ggr, ggr2, r, g, b, mapWidth, mapHeight);
        }

    //- Draw packets
    packet=pfirstGamePacket;
    while(packet!=NULL)
    {
        int x;
        int y;
        
        x=packet->x;
        y=packet->y;
        
        DrawBox(0, x, y, 150, 255, 0, mapWidth, mapHeight);
        
        packet=packet->next;
    }

    //- Draw enemies
    player=pfirstPlayer;
    while(player!=NULL)
    {
        int x;
        int y;
        
        x=player->x;
        y=player->y;
        
        if(player->id!=robotId)
            DrawBox(0, x, y, 150, 0, 0, mapWidth, mapHeight);
        
        player=player->next;
    }
    
    //- Draw us
    DrawBox(0, robotX, robotY, 0, 240, 240, mapWidth, mapHeight);
    

    //- Draw move board
    for(ggr=0;ggr<mapWidth;ggr++)
        for(ggr2=0;ggr2<mapHeight;ggr2++)
        {
            int point;
            int r, g, b;
            
            point=getMovePoint(ggr, ggr2);
            switch(point)
            {
                case 1:
                    r=0;
                    g=0;
                    b=120;
                    break;
                    
                case 0:
                    r=170;
                    g=170;
                    b=170;
                    break;
                default:
                    printf("move map: ('%c', %d)\n\n", point, point);
                    ERROR("Invalid point in DrawScreens");
            }
            
            DrawBox(1, ggr, ggr2, r, g, b, mapWidth, mapHeight);
        }

    //- Draw dist board
    for(ggr=0;ggr<mapWidth;ggr++)
        for(ggr2=0;ggr2<mapHeight;ggr2++)
        {
            int point;
            int r, g, b;
            
            point=getDistPoint(ggr, ggr2);
    

            r=(point/100)*50;
            g=(point/10)*20;
            b=10*point;
            
            DrawBox(2, ggr, ggr2, r, g, b, mapWidth, mapHeight);
        }

    //- Draw score board
    for(ggr=0;ggr<mapWidth;ggr++)
        for(ggr2=0;ggr2<mapHeight;ggr2++)
        {
            int point;
            int r, g, b;
            
            point=getScorePoint(ggr, ggr2);
    

            if(point>255*2)
            {
                r=point-255*2+100;
                g=255;
                b=255;
            }
            else if(point>255)
            {
                r=100;
                g=point-255;
                b=255;
            }
            else
            {
                r=100*((point>0)?1:0);
                g=0;
                b=point;
            }
            
            DrawBox(3, ggr, ggr2, r, g, b, mapWidth, mapHeight);
        }

    //- Draw packet board
    for(ggr=0;ggr<mapWidth;ggr++)
        for(ggr2=0;ggr2<mapHeight;ggr2++)
        {
            int point;
            int r, g, b;
            
            point=getPacketPoint(ggr, ggr2);
    

            r=0;
            g=10*point;
            b=10*point;
            
            DrawBox(4, ggr, ggr2, r, g, b, mapWidth, mapHeight);
        }

    //- Draw dest board
    for(ggr=0;ggr<mapWidth;ggr++)
        for(ggr2=0;ggr2<mapHeight;ggr2++)
        {
            int point;
            int r, g, b;
            
            point=getDestPoint(ggr, ggr2);
    

            r=0;
            g=150*point;
            b=0;
            
            DrawBox(5, ggr, ggr2, r, g, b, mapWidth, mapHeight);
        }

    //- Unlock screens
    for(ggr=0;ggr<SCREENS;ggr++)
        Sulock(screens[ggr]);

    ShowScreens();
}

void DrawBox(int screen, int x, int y, Uint8 r, Uint8 g, Uint8 b, int width, int height)
{
    int yswap=height-y-1;

    int x1=(SCREENWIDTH*x)/width;
    int y1=(SCREENHEIGHT*yswap)/height;
    int x2=(SCREENWIDTH*(x+1))/width;
    int y2=(SCREENHEIGHT*(yswap+1))/height;

    SDL_Rect rect;

    Uint32 color;
    
    
    rect.x=x1;
    rect.y=y1;
    rect.w=x2-x1;
    rect.h=y2-y1;

    color=SDL_MapRGB(screens[screen]->format, r, g, b);
    
    SDL_FillRect(screens[screen], &rect, color);
}

void ShowScreens()
{
    SDL_Rect src, dest;
    
    src.x=0;
    src.y=0;
    src.w=SCREENWIDTH;
    src.h=SCREENHEIGHT;
    
    dest.x=0;
    dest.y=0;
    dest.w=SCREENWIDTH;
    dest.h=SCREENHEIGHT;

    Slock(window);

    SDL_BlitSurface(screens[0], &src, window, &dest);

    dest.x=SCREENWIDTH;
    dest.y=0;
    SDL_BlitSurface(screens[3], &src, window, &dest);

    dest.x=0;
    dest.y=SCREENHEIGHT;
    SDL_BlitSurface(screens[5], &src, window, &dest);

    dest.x=SCREENWIDTH;
    dest.y=SCREENHEIGHT;
    SDL_BlitSurface(screens[4], &src, window, &dest);

    Sulock(window);
    SDL_Flip(window);
}

void Slock(SDL_Surface *screen)
{
  if ( SDL_MUSTLOCK(screen) )
  {
    if ( SDL_LockSurface(screen) < 0 )
    {
      return;
    }
  }
}

void Sulock(SDL_Surface *screen)
{
  if ( SDL_MUSTLOCK(screen) )
  {
    SDL_UnlockSurface(screen);
  }
}

#endif
